#include "AT898252.H"
#include "service.h"
#include "LCD.h"
#include "stdio.h"
#include "defines.h"
#include "dataflash.h"
#include "time.h"

extern char tm_sec;     /* seconds after the minute - [0,59] */
extern char tm_min;     /* minutes after the hour - [0,59] */
extern char tm_hour;    /* hours since midnight - [0,23] */
extern char tm_mday;    /* day of the month - [1,31] */
extern char tm_mon;     /* months since January - [1,12] */
extern int tm_year;    /* year */

#define DF_BUSY  P1_0
#define DF_CS	 P1_1
#define DF_WP	 P1_2
#define DF_RESET P1_3
#define SPI_PORT P1

#define WAITBUSY while (!(SPSR&0x80))
#define WAITOPEND while (!DF_BUSY)

#define PAGE_SIZE			264		// size of page in bytes
#define MAX_PAGE_OFF		263		// size of the page
#define LAST_OFFSET 		252		// last position, where 12-byte log record can be saved
#define LAST_PAGE			2047	// address of the last page
#define MAXLR_ON_PAGE		22		// max number of log records per page

int df_page_off;
int df_in_page_off;

unsigned char cData[12];

extern char iData[8];
extern char sz[17];
extern char cr;
extern char i;

void df_startop(void)
{
	SPCR=0x5D;	//01011101b
					    // frequency = fosc/16
				 	  // 
				    //     SPI Master Mode 3
				  // Interrupt disable, SPI Enable		
	DF_CS=0; 	// Select dataflash (make active)
}

// wait until DataFlash completes processing
void df_endop(void)
{
	DF_CS=1;
	WAITOPEND;
	SPCR=0x00;
}

void df_read_cur_page_to_buf_1(void)
{
	df_startop();

	SPDR=0x53;	//Main memory page to buffer 1 transfer
	WAITBUSY;
	SPDR=(char)(df_page_off>>7);
	WAITBUSY;
	SPDR=(char)(df_page_off<<1);
	WAITBUSY;
	SPDR=0;
	WAITBUSY;

	df_endop();
}

// decrement memory location if in data flash
void df_shift_left(char len)
{
	df_in_page_off-=len;

	if (df_in_page_off<0)		// switch to next page
	{
		df_page_off--;

		if (df_page_off<0)	// beginning of flash memory reached. Start from end 
			df_page_off=LAST_PAGE;

		df_in_page_off=LAST_OFFSET;
		df_read_cur_page_to_buf_1();
	}
}

// increment memory location in dataflash
char df_shift_right(char len)
{
	df_in_page_off+=len;
	if (df_in_page_off>LAST_OFFSET)		// switch to next page
	{
		df_page_off++;

		if (df_page_off>LAST_PAGE)	// end of flash memory reached. Start from beginning
			df_page_off=0;

		df_in_page_off=0;
		df_read_cur_page_to_buf_1();

		return 1;
	}
	return 0;
}

// function erases page, specified by df_page_off variable
void df_erase_page(void)
{
	df_in_page_off=0;

	df_startop();
	SPDR=0x82;	//Main memory page program through buffer 1
	WAITBUSY;
	SPDR=(char)(df_page_off>>7);
	WAITBUSY;
	SPDR=(char)(df_page_off<<1) | (char)(df_in_page_off>>8);
	WAITBUSY;
	SPDR=(char)df_in_page_off;
	WAITBUSY;

	while (df_in_page_off++!=PAGE_SIZE)	// now fill buffer with FF's
	{
		SPDR=0xFF;
		WAITBUSY;
	}

	df_endop();

	df_in_page_off=0;

	df_read_cur_page_to_buf_1();

}

// function reads from the dataflash at the offset specified by
// df_page_off and df_in_page_off and moves to the next/previous record
// function assumes that all records have the same length
// if shift=0 increment position
// if shift=1 decrement position
void df_read_flash_to_buff(char *pDest, char len, char shiftleft)
{
	//	page offset     		xxxx xPPP PPPP PPPP
	//  inpage offset   		xxxx xxxI IIII IIII
	//  address       RRRR PPPP PPPP PPPI IIII IIII
	df_startop();
	SPDR=0x54;	//buffer 1 read
	WAITBUSY;
	SPDR=0;		//reserved
	WAITBUSY;
	SPDR=(char)(df_in_page_off>>8);
	WAITBUSY;
	SPDR=(char)df_in_page_off;
	WAITBUSY;

	SPDR=0;	// don't care
	WAITBUSY;

	i=0;
	while (i<len)
	{
		SPDR=0;		//dummy value to initiate transfer
		WAITBUSY;
		pDest[i++]=SPDR;
	}

	df_endop();
	
	if (shiftleft)	// shift back
		df_shift_left(len);
	else	// shift forward
		df_shift_right(len);
	
}

// writes data to DF through buffer 1. Incerements offset counters
// if end of page is reached - next page is erased and page counter is inceremnted
// if end of flash is reached - counters are reset;
// length of the passed data can not exceed the rest space on the page, otherwise the DataFlash will
// automatically start to write exceeding data from the beginning of the page
void df_write_through_buffer1(char* pSrc, char len)
{
	//	page offset     		xxxx xPPP PPPP PPPP
	//  inpage offset   		xxxx xxxI IIII IIII
	//  address       RRRR PPPP PPPP PPPI IIII IIII
	df_startop();
	SPDR=0x82;	//Main memory page program through buffer 1
	WAITBUSY;
	SPDR=(char)(df_page_off>>7);
	WAITBUSY;
	SPDR=(char)(df_page_off<<1) | (char)(df_in_page_off>>8);
	WAITBUSY;
	SPDR=(char)df_in_page_off;
	WAITBUSY;

	i=0;
	while (i<len)
	{
		SPDR=pSrc[i++];
		WAITBUSY;
	}

	df_endop();
	
	if (df_shift_right(len))
		df_erase_page();
}

// call this function to search for the last record in the DF,
// initialize page and in-page offset counters.

void df_init_page_counters(void)
{
	df_page_off=0;
	df_in_page_off=0;
	df_read_cur_page_to_buf_1();
	cData[0]=0;

	while (cData[0]!=0xFF)
		df_read_flash_to_buff(cData,12,0);
		
	if (df_in_page_off+df_page_off)
		df_read_flash_to_buff(cData, 12,1); // shift one position left, as far as it is shifted one pos right
}

//init DataFlash subsystem
void df_init(void)
{
	SPI_PORT=0xFF; 	// Set all bits of SPI port to 1
	DF_RESET=0;		// reset DataFlash
	DF_WP=1;	// Write enable of first 256 pages
	DF_RESET=1; // Release reset
	df_init_page_counters(); // retrieve current values for page counters.
}
// View log
void ViewLog(void)
{
	char dir;
	df_init_page_counters();
	lcd_print("<* #> 0-exit");
	df_shift_left(12);
	while (1)
	{
		switch (readkey())
		{
			case 10:// read and scan left
				dir=1;
				break;
			case 11:	// read and scan right
				dir=0;
				break;
			case 0:
				df_init_page_counters();
				return;
			default:
				continue;
		}

		df_read_flash_to_buff(cData, 12,dir);

		if (cData[0]==0xFF)
			lcd_print("Empty");
		else
		{
			Byte2Hex (sz,cData,8);
			lcd_send_str(sz);
			time2tm(*((long*)&cData[8]));
			sprintf(sz,"%0d/%bd/%bd %bd:%bd:%bd",tm_year-2000,tm_mon,tm_mday,tm_hour,tm_min,tm_sec);
			lcd_send_str(sz);
		}
	}
	df_init_page_counters();
}
// Erase first page in data flash
// After this operation LOG will be written from the very beginning of DF address space
void EraseFirstPage(void)
{
	df_page_off=0;
	df_erase_page();
	xmessagebox("Ok");
}
